"""Yet Another Niconico-douga Command-line Interface."""

#  Copyright 2015-2024 Masayuki Yamamoto
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from importlib.metadata import PackageNotFoundError, version

__author__ = "Masayuki Yamamoto"
__copyright__ = "Copyright 2015-2024 Yamamoto Masayuki"
__description__ = __doc__
__license__ = "Apache 2.0"

try:
    __version__ = version(__name__)
except PackageNotFoundError:
    __version__ = "unknown"
